@@ -5,6 +5,13 @@
 #include "vector.h"
 #endif
 
+enum devtypes {
+	DEV_NONE,
+	DEV_DEVT,
+	DEV_DEVNODE,
+	DEV_DEVMAP
+};
+
 struct mpentry {
 	int selector_args;
 	int pgpolicy;
@@ -15,20 +22,6 @@ struct mpentry {
 	char * alias;
 };
 
-struct hwentry {
-	int selector_args;
-	int pgpolicy;
-	int checker_index;
-
-	char * vendor;
-	char * product;
-	char * selector;
-	char * getuid;
-	char * getprio;
-	char * features;
-	char * hwhandler;
-};
-
 struct config {
 	int verbosity;
 	int dry_run;
@@ -38,6 +31,8 @@ struct config {
 	int with_sysfs;
 	int default_selector_args;
 	int default_pgpolicy;
+	int dev_type;
+	int minio;
 
 	char * dev;
 	char * udev_dir;
