@@ -13,7 +13,11 @@
 #include <sysfs/libsysfs.h>
 #include <memory.h>
 #include <callout.h>
+#include <util.h>
+#include <vector.h>
+#include <structs.h>
 
+#include "sysfs_devinfo.h"
 #include "devinfo.h"
 #include "sg_include.h"
 #include "debug.h"
@@ -23,16 +27,6 @@
 #define readattr(a,b) \
 	sysfs_read_attribute_value(a, b, sizeof(b))
 
-void
-basename (char * str1, char * str2)
-{
-        char *p = str1 + (strlen(str1) - 1);
- 
-        while (*--p != '/')
-                continue;
-        strcpy(str2, ++p);
-}
-
 static int
 opennode (char * devt, int mode)
 {
@@ -119,6 +113,7 @@ devt2devname (char *devname, char *devt)
 	char block_path[FILE_NAME_SIZE];
 	char attr_path[FILE_NAME_SIZE];
 	char attr_value[16];
+	int len;
 
 	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
 		fprintf(stderr, "feature available with sysfs only\n");
@@ -141,7 +136,13 @@ devt2devname (char *devname, char *devt)
 		sysfs_read_attribute_value(attr_path, attr_value,
 					   sizeof(attr_value));
 
-		if (!strncmp(attr_value, devt, strlen(devt))) {
+		len = strlen(attr_value);
+
+		/* discard newline */
+		if (len > 1) len--;
+
+		if (strlen(devt) == len &&
+		    strncmp(attr_value, devt, len) == 0) {
 			if(safe_sprintf(attr_path, "%s/%s",
 					block_path, devp->name)) {
 				fprintf(stderr, "attr_path too small\n");
@@ -149,6 +150,7 @@ devt2devname (char *devname, char *devt)
 			}
 			sysfs_get_name_from_path(attr_path, devname,
 						 FILE_NAME_SIZE);
+			sysfs_close_directory(sdir);
 			return 0;
 		}
 	}
@@ -240,84 +242,36 @@ sysfs_devinfo(struct path * curpath)
 	char attr_path[FILE_NAME_SIZE];
 	char attr_buff[FILE_NAME_SIZE];
 	char sysfs_path[FILE_NAME_SIZE];
-	struct stat buf;
 
 	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
 		fprintf(stderr, "need sysfs mounted : out\n");
 		return 1;
 	}
-	if (safe_sprintf(attr_path, "%s/block/%s", sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
-		return 1;
-	}
-	if(stat(attr_path, &buf))
-		return 1;
 
-	/*
-	 * vendor string
-	 */
-	if(safe_sprintf(attr_path, "%s/block/%s/device/vendor",
-			sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
+	if (sysfs_get_vendor(sysfs_path, curpath->dev,
+			     curpath->vendor_id, SCSI_VENDOR_SIZE))
 		return 1;
-	}
-	if (0 > readattr(attr_path, attr_buff))
-		return 1;
-	memcpy(curpath->vendor_id, attr_buff, 8);
 	dbg("vendor = %s", curpath->vendor_id);
 
-	/*
-	 * model string
-	 */
-	if(safe_sprintf(attr_path, "%s/block/%s/device/model",
-		sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
+	if (sysfs_get_model(sysfs_path, curpath->dev,
+			    curpath->product_id, SCSI_PRODUCT_SIZE))
 		return 1;
-	}
-	if (0 > readattr(attr_path, attr_buff))
-			return 1;
-	memcpy(curpath->product_id, attr_buff, 16);
 	dbg("product = %s", curpath->product_id);
 
-	/*
-	 * revision string
-	 */
-	if(safe_sprintf(attr_path, "%s/block/%s/device/rev",
-		sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
+	if (sysfs_get_rev(sysfs_path, curpath->dev,
+			  curpath->rev, SCSI_REV_SIZE))
 		return 1;
-	}
-	if (0 > readattr(attr_path, attr_buff))
-		return 1;
-	memcpy(curpath->rev, attr_buff, 4);
 	dbg("rev = %s", curpath->rev);
 
-	/*
-	 * bdev major:minor string
-	 */
-	if(safe_sprintf(attr_path, "%s/block/%s/dev",
-		sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
+	if (sysfs_get_dev(sysfs_path, curpath->dev,
+			  curpath->dev_t, BLK_DEV_SIZE))
 		return 1;
-	}
-	if (0 > readattr(attr_path, attr_buff))
-		return 1;
-	if (strlen(attr_buff) > 1)
-		strncpy(curpath->dev_t, attr_buff,
-			strlen(attr_buff) - 1);
 	dbg("dev_t = %s", curpath->dev_t);
 
-	/*
-	 * size
-	 */
-	if(safe_sprintf(attr_path, "%s/block/%s/size",
-		sysfs_path, curpath->dev)) {
-		fprintf(stderr, "attr_path too small\n");
+	curpath->size = sysfs_get_size(sysfs_path, curpath->dev);
+
+	if (curpath->size == 0)
 		return 1;
-	}
-	if (0 > readattr(attr_path, attr_buff))
-		return 1;
-	curpath->size = strtoul(attr_buff, NULL, 0);
 	dbg("size = %lu", curpath->size);
 
 	/*
@@ -354,9 +308,7 @@ sysfs_devinfo(struct path * curpath)
 		fprintf(stderr, "attr_path too small\n");
 		return 1;
 	}
-	if (0 > readattr(attr_path, attr_buff))
-		return 1;
-	if (strlen(attr_buff) > 0)
+	if (0 <= readattr(attr_path, attr_buff) && strlen(attr_buff) > 0)
 		strncpy(curpath->tgt_node_name, attr_buff,
 			strlen(attr_buff) - 1);
 	dbg("tgt_node_name = %s", curpath->tgt_node_name);
@@ -371,7 +323,7 @@ apply_format (char * string, int maxsize, struct path * pp)
 	char * dst;
 	char * p;
 	int len;
-	int free;
+	int myfree;
 
 	if (!string)
 		return NULL;
@@ -383,16 +335,20 @@ apply_format (char * string, int maxsize, struct path * pp)
 
 	p = dst;
 	pos = strchr(string, '%');
-	free = maxsize;
+	myfree = maxsize;
 
-	if (!pos)
-		return string;
+	if (!pos) {
+		strcpy(dst, string);
+		return dst;
+	}
 
 	len = (int) (pos - string) + 1;
-	free -= len;
+	myfree -= len;
 
-	if (free < 2)
+	if (myfree < 2) {
+		free(dst);
 		return NULL;
+	}
 
 	snprintf(p, len, "%s", string);
 	p += len - 1;
@@ -401,20 +357,24 @@ apply_format (char * string, int maxsize, struct path * pp)
 	switch (*pos) {
 	case 'n':
 		len = strlen(pp->dev) + 1;
-		free -= len;
+		myfree -= len;
 
-		if (free < 2)
+		if (myfree < 2) {
+			free(dst);
 			return NULL;
+		}
 
 		snprintf(p, len, "%s", pp->dev);
 		p += len - 1;
 		break;
 	case 'd':
 		len = strlen(pp->dev_t) + 1;
-		free -= len;
+		myfree -= len;
 
-		if (free < 2)
+		if (myfree < 2) {
+			free(dst);
 			return NULL;
+		}
 
 		snprintf(p, len, "%s", pp->dev_t);
 		p += len - 1;
@@ -428,10 +388,12 @@ apply_format (char * string, int maxsize, struct path * pp)
 		return dst;
 
 	len = strlen(pos) + 1;
-	free -= len;
+	myfree -= len;
 
-	if (free < 2)
+	if (myfree < 2) {
+		free(dst);
 		return NULL;
+	}
 
 	snprintf(p, len, "%s", pos);
 	dbg("reformated callout = %s", dst);
@@ -478,6 +440,7 @@ devinfo (struct path *pp)
 	else if (execute_program(buff, prio, 16)) {
 		dbg("error calling out %s", buff);
 		pp->priority = 1;
+		free(buff);
 	} else
 		pp->priority = atoi(prio);
 
@@ -488,10 +451,13 @@ devinfo (struct path *pp)
 	 */
 	select_getuid(pp);
 	buff = apply_format(pp->getuid, CALLOUT_MAX_SIZE, pp);
-
-	if (buff && execute_program(buff, pp->wwid, WWID_SIZE) == 0) {
-		dbg("uid = %s (callout)", pp->wwid);
-		return 0;
+	if (buff) {
+		if (execute_program(buff, pp->wwid, WWID_SIZE) == 0) {
+			free(buff);
+			dbg("uid = %s (callout)", pp->wwid);
+			return 0;
+		}
+		free(buff);
 	}
 
 	/*
