@@ -41,6 +41,7 @@
 #include <util.h>
 #include <defaults.h>
 #include <structs.h>
+#include <dmparser.h>
 
 #include "main.h"
 #include "devinfo.h"
@@ -48,12 +49,8 @@
 #include "pgpolicies.h"
 #include "dict.h"
 #include "debug.h"
-#include "dmparser.h"
 #include "propsel.h"
 
-/* helpers */
-#define argis(x) if (0 == strcmp(x, argv[i]))
-
 static int
 filepresent (char * run) {
 	struct stat buf;
@@ -88,7 +85,7 @@ get_pathvec_sysfs (vector pathvec)
 	 * if called from /etc/dev.d or pathcheckers, only consider the paths
 	 * that relate to the device pointed by conf->dev
 	 */
-	if (conf->dev && filepresent(conf->dev)) {
+	if (conf->dev_type == DEV_DEVNODE) {
 		dbg("limited scope = %s", conf->dev);
 		curpath = zalloc(sizeof (struct path));
 		basename(conf->dev, curpath->dev);
@@ -104,7 +101,7 @@ get_pathvec_sysfs (vector pathvec)
 	 * if devt specified on the cmd line,
 	 * only consider affiliated paths
 	 */
-	if (conf->dev && !devt2devname(buff, conf->dev)) {
+	if (conf->dev_type == DEV_DEVT && !devt2devname(buff, conf->dev)) {
 		dbg("limited scope = %s", conf->dev);
 		curpath = zalloc(sizeof (struct path));
 		if(safe_sprintf(curpath->dev, "%s", buff)) {
@@ -149,8 +146,9 @@ get_pathvec_sysfs (vector pathvec)
 			exit(1);
 		}
 		if (devinfo(curpath)) {
+			fprintf(stderr, "pb getting path info, free %s\n",
+					curpath->dev);
 			free (curpath);
-			fprintf(stderr, "pb getting path info, free\n");
 			continue;
 		}
 		if (memcmp(empty_buff, refwwid, WWID_SIZE) != 0 && 
@@ -159,6 +157,8 @@ get_pathvec_sysfs (vector pathvec)
 			free(curpath);
 			continue;
 		}
+		curpath->hwe = find_hwe(conf->hwtable, curpath->vendor_id,
+						      curpath->product_id);
 		vector_alloc_slot(pathvec);
 		vector_set_slot(pathvec, curpath);
 	}
@@ -253,16 +253,6 @@ print_all_paths (vector pathvec)
 		print_path(pp, PRINT_PATH_ALL);
 	}
 }
-
-static void
-print_all_maps (vector mp)
-{
-	int i;
-	struct multipath * mpp;
-
-	vector_foreach_slot (mp, mpp, i)
-		print_map(mpp);
-}
 #endif
 
 static void
@@ -356,153 +346,6 @@ print_mp (struct multipath * mpp)
 	printf("\n");
 }
 
-static void
-print_all_mp (vector mp)
-{
-	int k;
-	struct multipath * mpp;
-
-	if (conf->verbosity == 0)
-		return;
-
-	vector_foreach_slot (mp, mpp, k) 
-		print_mp(mpp);
-}
-
-static void
-coalesce_paths (vector mp, vector pathvec)
-{
-	int k, i;
-	char empty_buff[WWID_SIZE];
-	struct multipath * mpp;
-	struct path * pp1;
-	struct path * pp2;
-
-	memset(empty_buff, 0, WWID_SIZE);
-
-	vector_foreach_slot (pathvec, pp1, k) {
-		/* skip this path for some reason */
-
-		/* 1. if path has no unique id */
-		if (memcmp(empty_buff, pp1->wwid, WWID_SIZE) == 0)
-			continue;
-
-		/* 2. if path already coalesced */
-		if (pp1->mpp)
-			continue;
-
-		/*
-		 * at this point, we know we really got a new mp
-		 */
-		mpp = zalloc(sizeof(struct multipath));
-		pp1->mpp = mpp;
-		strcpy(mpp->wwid, pp1->wwid);
-		mpp->size = pp1->size;
-
-		mpp->mpe = find_mp(pp1->wwid);
-		mpp->hwe = find_hw(conf->hwtable,
-				   pp1->vendor_id, pp1->product_id);
-
-		mpp->paths = vector_alloc();
-		vector_alloc_slot (mpp->paths);
-		vector_set_slot (mpp->paths, pp1);
-
-		for (i = k + 1; i < VECTOR_SIZE(pathvec); i++) {
-			pp2 = VECTOR_SLOT(pathvec, i);
-
-			if (strcmp(pp1->wwid, pp2->wwid))
-				continue;
-			
-			pp2->mpp = mpp;
-
-			if (pp2->size != mpp->size) {
-				/*
-				 * ouch, avoid feeding that to the DM
-				 */
-				dbg("path size mismatch : discard %s",
-				     mpp->wwid);
-				mpp->action = ACT_NOTHING;
-			}
-			vector_alloc_slot(mpp->paths);
-			vector_set_slot(mpp->paths, VECTOR_SLOT(pathvec, i));
-		}
-		if (mpp) {
-			vector_alloc_slot(mp);
-			vector_set_slot(mp, mpp);
-		}
-	}
-}
-
-static int
-dm_switchgroup(char * mapname, int index)
-{
-	int r = 0;
-	struct dm_task *dmt;
-	char str[24];
-
-	if (!(dmt = dm_task_create(DM_DEVICE_TARGET_MSG)))
-		return 0;
-
-	if (!dm_task_set_name(dmt, mapname))
-		goto out;
-
-	if (!dm_task_set_sector(dmt, 0))
-		goto out;
-
-	snprintf(str, 24, "switch_group %i\n", index);
-	dbg("message %s 0 %s", mapname, str);
-
-	if (!dm_task_set_message(dmt, str))
-		goto out;
-
-	if (!dm_task_run(dmt))
-		goto out;
-
-	r = 1;
-
-	out:
-	dm_task_destroy(dmt);
-
-	return r;
-}
-static int
-dm_reinstate(char * mapname, char * path)
-{
-	int r = 0;
-	int sz;
-	struct dm_task *dmt;
-	char *str;
-
-	if (!(dmt = dm_task_create(DM_DEVICE_TARGET_MSG)))
-		return 0;
-
-	if (!dm_task_set_name(dmt, mapname))
-		goto out;
-
-	if (!dm_task_set_sector(dmt, 0))
-		goto out;
-
-	sz = strlen(path) + 16;
-	str = zalloc(sz);
-
-	snprintf(str, sz, "reinstate_path %s\n", path);
-
-	if (!dm_task_set_message(dmt, str))
-		goto out;
-
-	free(str);
-
-	if (!dm_task_run(dmt))
-		goto out;
-
-	r = 1;
-
-	out:
-	dm_task_destroy(dmt);
-
-	return r;
-}
-
 /*
  * Transforms the path group vector into a proper device map string
  */
@@ -541,7 +384,8 @@ assemble_map (struct multipath * mp)
 		freechar -= shift;
 
 		vector_foreach_slot (pgp->paths, pp, j) {
-			shift = snprintf(p, freechar, " %s 1000" ,pp->dev_t);
+			shift = snprintf(p, freechar, " %s %d",
+					 pp->dev_t, conf->minio);
 			if (shift >= freechar) {
 				fprintf(stderr, "mp->params too small\n");
 				exit(1);
@@ -555,6 +399,9 @@ assemble_map (struct multipath * mp)
 		exit(1);
 	}
 	snprintf(p, 1, "\n");
+#if DEBUG
+	print_map(mp);
+#endif
 }
 
 static int
@@ -586,7 +433,6 @@ setup_map (struct multipath * mpp)
 	select_selector(mpp);
 	select_features(mpp);
 	select_hwhandler(mpp);
-	select_alias(mpp);
 
 	/*
 	 * apply selected grouping policy to valid paths
@@ -639,6 +485,21 @@ setup_map (struct multipath * mpp)
 	return 0;
 }
 
+static int
+pathcount (struct multipath * mpp, int state)
+{
+	struct pathgroup *pgp;
+	struct path *pp;
+	int i, j;
+	int count = 0;
+
+	vector_foreach_slot (mpp->pg, pgp, i)
+		vector_foreach_slot (pgp->paths, pp, j)
+			if (pp->state == state)
+				count++;
+	return count;
+}
+
 /*
  * detect if a path is in the map we are about to create but not in the
  * current one (triggers a valid reload)
@@ -704,58 +565,94 @@ pgcmp (struct multipath * mpp, struct multipath * cmpp)
 static void
 select_action (struct multipath * mpp, vector curmp)
 {
-	int i;
 	struct multipath * cmpp;
 
-	vector_foreach_slot (curmp, cmpp, i) {
-		if (strncmp(cmpp->alias, mpp->alias, strlen(mpp->alias)))
-			continue;
+	cmpp = find_mp(curmp, mpp->alias);
 
-		if (cmpp->size != mpp->size) {
-			dbg("size different than current");
-			mpp->action = ACT_RELOAD;
-			return;
-		}
-		if (strncmp(cmpp->features, mpp->features,
-			    strlen(mpp->features))) {
-			dbg("features different than current");
-			mpp->action =  ACT_RELOAD;
-			return;
-		}
-		if (strncmp(cmpp->hwhandler, mpp->hwhandler,
-			    strlen(mpp->hwhandler))) {
-			dbg("hwhandler different than current");
-			mpp->action = ACT_RELOAD;
-			return;
-		}
-		if (strncmp(cmpp->selector, mpp->selector,
-			    strlen(mpp->selector))) {
-			dbg("selector different than current");
-			mpp->action = ACT_RELOAD;
-			return;
-		}
-		if (VECTOR_SIZE(cmpp->pg) != VECTOR_SIZE(mpp->pg)) {
-			dbg("different number of PG");
-			mpp->action = ACT_RELOAD;
-			return;
-		}
-		if (pgcmp2(mpp, cmpp)) {
-			dbg("different path group topology");
-			mpp->action = ACT_RELOAD;
-			return;
-		}
-		if (cmpp->nextpg != mpp->nextpg) {
-			dbg("nextpg different than current");
-			mpp->action = ACT_SWITCHPG;
-			return;
-		}
+	if (!cmpp) {
+		mpp->action = ACT_CREATE;
+		return;
+	}
+	if (pathcount(mpp, PATH_UP) == 0) {
+		dbg("no good path");
 		mpp->action = ACT_NOTHING;
 		return;
 	}
-	mpp->action = ACT_CREATE;
+	if (cmpp->size != mpp->size) {
+		dbg("size different than current");
+		mpp->action = ACT_RELOAD;
+		return;
+	}
+	if (strncmp(cmpp->features, mpp->features,
+		    strlen(mpp->features))) {
+		dbg("features different than current");
+		mpp->action =  ACT_RELOAD;
+		return;
+	}
+	if (strncmp(cmpp->hwhandler, mpp->hwhandler,
+		    strlen(mpp->hwhandler))) {
+		dbg("hwhandler different than current");
+		mpp->action = ACT_RELOAD;
+		return;
+	}
+	if (strncmp(cmpp->selector, mpp->selector,
+		    strlen(mpp->selector))) {
+		dbg("selector different than current");
+		mpp->action = ACT_RELOAD;
+		return;
+	}
+	if (VECTOR_SIZE(cmpp->pg) != VECTOR_SIZE(mpp->pg)) {
+		dbg("different number of PG");
+		mpp->action = ACT_RELOAD;
+		return;
+	}
+	if (pgcmp2(mpp, cmpp)) {
+		dbg("different path group topology");
+		mpp->action = ACT_RELOAD;
+		return;
+	}
+	if (cmpp->nextpg != mpp->nextpg) {
+		dbg("nextpg different than current");
+		mpp->action = ACT_SWITCHPG;
+		return;
+	}
+	mpp->action = ACT_NOTHING;
 	return;
 }
 
+static int
+dm_switchgroup(char * mapname, int index)
+{
+	int r = 0;
+	struct dm_task *dmt;
+	char str[24];
+
+	if (!(dmt = dm_task_create(DM_DEVICE_TARGET_MSG)))
+		return 0;
+
+	if (!dm_task_set_name(dmt, mapname))
+		goto out;
+
+	if (!dm_task_set_sector(dmt, 0))
+		goto out;
+
+	snprintf(str, 24, "switch_group %i\n", index);
+	dbg("message %s 0 %s", mapname, str);
+
+	if (!dm_task_set_message(dmt, str))
+		goto out;
+
+	if (!dm_task_run(dmt))
+		goto out;
+
+	r = 1;
+
+	out:
+	dm_task_destroy(dmt);
+
+	return r;
+}
+
 static int
 reinstate_paths (struct multipath * mpp)
 {
@@ -796,7 +693,8 @@ domap (struct multipath * mpp)
 		dm_switchgroup(mpp->alias, mpp->nextpg);
 		/*
 		 * we may have avoided reinstating paths because there where in
-		 * active or disabled PG. Now that the topology has changed, retry.
+		 * active or disabled PG. Now that the topology has changed,
+		 * retry.
 		 */
 		reinstate_paths(mpp);
 		return 0;
@@ -833,6 +731,83 @@ domap (struct multipath * mpp)
 	return r;
 }
 
+static void
+coalesce_paths (vector curmp, vector pathvec)
+{
+	int k, i;
+	char empty_buff[WWID_SIZE];
+	struct multipath * mpp;
+	struct path * pp1;
+	struct path * pp2;
+
+	memset(empty_buff, 0, WWID_SIZE);
+
+	vector_foreach_slot (pathvec, pp1, k) {
+		/* skip this path for some reason */
+
+		/* 1. if path has no unique id */
+		if (memcmp(empty_buff, pp1->wwid, WWID_SIZE) == 0)
+			continue;
+
+		/* 2. if path already coalesced */
+		if (pp1->mpp)
+			continue;
+
+		/*
+		 * at this point, we know we really got a new mp
+		 */
+		mpp = zalloc(sizeof(struct multipath));
+
+		mpp->mpe = find_mpe(pp1->wwid);
+		mpp->hwe = pp1->hwe;
+		select_alias(mpp);
+
+		if (conf->dev_type == DEV_DEVMAP &&
+		    (mpp->alias &&
+		    0 != strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE)) &&
+		    0 != strncmp(pp1->wwid, conf->dev, FILE_NAME_SIZE)) {
+			free(mpp);
+			continue;
+		}
+		pp1->mpp = mpp;
+		strcpy(mpp->wwid, pp1->wwid);
+		mpp->size = pp1->size;
+
+		mpp->paths = vector_alloc();
+		vector_alloc_slot (mpp->paths);
+		vector_set_slot (mpp->paths, pp1);
+
+		for (i = k + 1; i < VECTOR_SIZE(pathvec); i++) {
+			pp2 = VECTOR_SLOT(pathvec, i);
+
+			if (strcmp(pp1->wwid, pp2->wwid))
+				continue;
+			
+			pp2->mpp = mpp;
+
+			if (pp2->size != mpp->size) {
+				/*
+				 * ouch, avoid feeding that to the DM
+				 */
+				dbg("path size mismatch : discard %s",
+				     mpp->wwid);
+				mpp->action = ACT_NOTHING;
+			}
+			vector_alloc_slot(mpp->paths);
+			vector_set_slot(mpp->paths, VECTOR_SLOT(pathvec, i));
+		}
+		if (mpp) {
+			setup_map(mpp);
+
+			if (mpp->action == ACT_UNDEF)
+				select_action(mpp, curmp);
+
+			domap(mpp);
+			free_multipath(mpp);
+		}
+	}
+}
+
 static int
 dm_get_maps (vector mp, char * type)
 {
@@ -945,46 +920,62 @@ usage (char * progname)
 	exit(1);
 }
 
-int try_lock (char * file)
+static int
+get_current_paths (struct multipath * mpp, vector pathvec)
 {
-	int fd;
-	struct flock fl;
+	int i, j;
+	struct pathgroup * pgp;
+	struct path * pp;
 
-	/*
-	 * create the file to lock if it does not exist
-	 */
-	fd = open(file, O_CREAT|O_RDWR);
-
-	if (fd < 0) {
-		fprintf(stderr, "can't create runfile\n");
-		exit(1);
+	vector_foreach_slot (mpp->pg, pgp, i) {
+		vector_foreach_slot (pgp->paths, pp, j) {
+			if (strlen(pp->dev) == 0) {
+				devt2devname(pp->dev, pp->dev_t);
+				devinfo(pp);
+			}
+			if (pp->state == PATH_UNCHECKED)
+				pp->state = pp->checkfn(pp->dev_t, NULL, NULL);
+		}
 	}
-	fl.l_type = F_WRLCK;
-	fl.l_whence = 0;
-	fl.l_start = 0;
-	fl.l_len = 0;
+	return 0;
+}
 
-	/*
-	 * set a max wait time
-	 */
-	alarm(2);
-	
-	if (fcntl(fd, F_SETLKW, &fl) == -1) {
-		fprintf(stderr, "can't take a write lease on %s\n", file);
-		return 1;
+static int
+get_current_mp (vector curmp, vector pathvec)
+{
+	int i;
+	struct multipath * mpp;
+
+	dm_get_maps(curmp, DEFAULT_TARGET);
+
+	vector_foreach_slot (curmp, mpp, i) {
+		if (conf->dev_type == DEV_DEVMAP && (
+		    (mpp->alias &&
+		    0 != strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE)) &&
+		    0 != strncmp(mpp->wwid, conf->dev, FILE_NAME_SIZE)))
+			continue;
+
+		dbg("params = %s", mpp->params);
+		dbg("status = %s", mpp->status);
+		disassemble_map(pathvec, mpp->params, mpp);
+		get_current_paths(mpp, pathvec);
+		disassemble_status(mpp->status, mpp);
+
+		if (conf->list)
+			print_mp(mpp);
+
+		if (!conf->dry_run)
+			reinstate_paths(mpp);
 	}
-	alarm(0);
-
 	return 0;
 }
 
 int
 main (int argc, char *argv[])
 {
-	vector mp, curmp;
+	vector curmp;
 	vector pathvec;
-	struct multipath * mpp;
-	int k;
+	int i;
 	int arg;
 	extern char *optarg;
 	extern int optind;
@@ -994,15 +985,6 @@ main (int argc, char *argv[])
 		exit(1);
 	}
 
-	/*
-	 * Don't run in parallel
-	 * ie, acquire a F_WRLCK-type lock on DEFAULT_RUNFILE
-	 */
-	if (try_lock(DEFAULT_RUNFILE)) {
-		fprintf(stderr, "waited for to long. exiting\n");
-		exit(1);
-	}
-	
 	/*
 	 * alloc config struct
 	 */
@@ -1017,6 +999,7 @@ main (int argc, char *argv[])
 	conf->pgpolicy_flag = 0;	/* do not override defaults */
 	conf->signal = 1;		/* 1 == Send a signal to multipathd */
 	conf->dev = NULL;
+	conf->dev_type = DEV_NONE;
 	conf->default_selector = NULL;
 	conf->default_selector_args = 0;
 	conf->default_pgpolicy = 0;
@@ -1025,6 +1008,7 @@ main (int argc, char *argv[])
 	conf->blist = NULL;
 	conf->default_features = NULL;
 	conf->default_hwhandler = NULL;
+	conf->minio = 1000;
 
 	while ((arg = getopt(argc, argv, ":qdlFSi:v:p:")) != EOF ) {
 		switch(arg) {
@@ -1073,16 +1057,23 @@ main (int argc, char *argv[])
 	if (optind<argc) {
 		conf->dev = zalloc(FILE_NAME_SIZE);
 		strncpy(conf->dev, argv[optind], FILE_NAME_SIZE);
+
+		if (filepresent(conf->dev))
+			conf->dev_type = DEV_DEVNODE;
+		else if (sscanf(conf->dev, "%d:%d", &i, &i) == 2)
+			conf->dev_type = DEV_DEVT;
+		else
+			conf->dev_type = DEV_DEVMAP;
+
 	}
 
 	/*
 	 * allocate core vectors to store paths and multipaths
 	 */
-	mp = vector_alloc();
 	curmp = vector_alloc();
 	pathvec = vector_alloc();
 
-	if (!mp || !curmp || !pathvec) {
+	if (!curmp || !pathvec) {
 		fprintf(stderr, "can not allocate memory\n");
 		exit(1);
 	}
@@ -1123,72 +1114,36 @@ main (int argc, char *argv[])
 		conf->default_hwhandler = DEFAULT_HWHANDLER;
 
 	/*
-	 * if we have a blacklisted device parameterexit early
+	 * if we have a blacklisted device parameter, exit early
 	 */
 	if (conf->dev && blacklist(conf->blist, conf->dev))
 		exit(0);
 	
 	/*
-	 * get a path list and group them as multipaths
+	 * get a path list
 	 */
 	if (get_pathvec_sysfs(pathvec) || VECTOR_SIZE(pathvec) == 0)
 		exit(1);
 
-	dm_get_maps(curmp, DEFAULT_TARGET);
-
 	if (VECTOR_SIZE(pathvec) == 0 && conf->verbosity > 0) {
 		fprintf(stdout, "no path found\n");
 		exit(0);
 	}
-
-	vector_foreach_slot (curmp, mpp, k) {
-		dbg("params = %s", mpp->params);
-		dbg("status = %s", mpp->status);
-		disassemble_map(pathvec, mpp->params, mpp);
-		disassemble_status(mpp->status, mpp);
-
-		if (!conf->dry_run)
-			reinstate_paths(mpp);
-	}
-	if (conf->list) {
-		print_all_mp(curmp);
-		goto out;
-	}
-
-	coalesce_paths(mp, pathvec);
-
-	/*
-	 * conf->dev can be a mapname
-	 * if so, only reconfigure this map
-	 */
-	vector_foreach_slot (mp, mpp, k) {
-		if (conf->dev && (
-		    (mpp->alias &&
-		    0 == strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE)) ||
-		    0 == strncmp(mpp->wwid, conf->dev, FILE_NAME_SIZE))) {
-			setup_map(mpp);
-
-			if (mpp->action == 0)
-				select_action(mpp, curmp);
-			domap(mpp);
-			goto out;
-		}
-	}
-
-	vector_foreach_slot (mp, mpp, k) {
-		setup_map(mpp);
-
-		if (mpp->action == 0)
-			select_action(mpp, curmp);
-		domap(mpp);
-	}
-
 #if DEBUG
 	fprintf(stdout, "#\n# all paths :\n#\n");
 	print_all_paths(pathvec);
-	fprintf(stdout, "#\n# device maps :\n#\n");
-	print_all_maps(mp);
 #endif
+
+	get_current_mp(curmp, pathvec);
+
+	if (conf->list)
+		goto out;
+
+	/*
+	 * group the paths as multipaths
+	 */
+	coalesce_paths(curmp, pathvec);
+
 out:
 	/*
 	 * signal multipathd that new devmaps may have come up
