@@ -31,36 +31,34 @@
 /*
  * libmultipath
  */
+#include <sysfs_devinfo.h>
 #include <parser.h>
 #include <vector.h>
 #include <memory.h>
 #include <callout.h>
 #include <safe_printf.h>
+#include <blacklist.h>
+#include <hwtable.h>
+#include <defaults.h>
+#include <structs.h>
+#include <dmparser.h>
+#include <devmapper.h>
 
-#include "hwtable.h"
+#include "main.h"
 #include "dict.h"
 #include "devinfo.h"
 #include "copy.h"
 #include "clone_platform.h"
 
-#define MAXPATHS 1024
-#define MAXMAPS 256
-#define FILENAMESIZE 256
-#define MAPNAMESIZE 64
-#define TARGETTYPESIZE 16
-#define PARAMSSIZE 2048
-#define DEV_T_SIZE 32
+#define FILE_NAME_SIZE 256
 #define CMDSIZE 160
 
-#define PIDFILE "/var/run/multipathd.pid"
-#define CONFIGFILE "/etc/multipath.conf"
 #define CALLOUT_DIR "/var/cache/multipathd"
 
 #ifndef LOGLEVEL
 #define LOGLEVEL 5
 #endif
 
-#define MATCH(x, y) strncmp(x, y, strlen(x)) == 0
 #define LOG_MSG(a,b) \
 	if (strlen(a)) { \
 		syslog(LOG_WARNING, "%s: %s", b, a); \
@@ -77,13 +75,6 @@ pthread_cond_t *event;
 /*
  * structs
  */
-struct path {
-	char dev_t[DEV_T_SIZE];
-	int state;
-	int (*checkfn) (char *, char *, void **);
-	void * checker_context;
-};
-
 struct paths {
 	pthread_mutex_t *lock;
 	vector pathvec;
@@ -93,23 +84,13 @@ struct event_thread {
 	pthread_t *thread;
 	pthread_mutex_t *waiter_lock;
 	int event_nr;
-	char mapname[MAPNAMESIZE];
+	char mapname[WWID_SIZE];
+	struct paths *allpaths;
 };
 
 /*
  * helpers functions
  */
-static int
-blacklist (char * dev) {
-	int i;
-	char *p;
-
-	vector_foreach_slot (blist, p, i)
-		if (memcmp(dev, p, strlen(p)) == 0)
-			return 1;
-	return 0;
-}
-
 static void
 strvec_free (vector vec)
 {
@@ -126,11 +107,11 @@ strvec_free (vector vec)
 static int
 select_checkfn(struct path *pp, char *devname)
 {
-	char vendor[8];
-	char product[16];
-	char rev[4];
+	char vendor[SCSI_VENDOR_SIZE];
+	char product[SCSI_PRODUCT_SIZE];
+	char rev[SCSI_REV_SIZE];
 	char checker_name[CHECKER_NAME_SIZE];
-	int i, r;
+	int r;
 	struct hwentry * hwe;
 
 	/*
@@ -144,19 +125,17 @@ select_checkfn(struct path *pp, char *devname)
 		syslog(LOG_ERR, "can not get scsi strings");
 		return r;
 	}
+	hwe = find_hwe(hwtable, vendor, product);
 
-	vector_foreach_slot (hwtable, hwe, i) {
-		if (MATCH(hwe->vendor, vendor) &&
-		    MATCH(hwe->product, product) &&
-		    hwe->checker_index > 0) {
-			get_checker_name(checker_name, hwe->checker_index);
-			syslog(LOG_INFO, "set %s path checker for %s",
-			     checker_name, devname);
-			pp->checkfn = get_checker_addr(hwe->checker_index);
-			return 0;
-		}
+	if (hwe && hwe->checker_index > 0) {
+		get_checker_name(checker_name, hwe->checker_index);
+		syslog(LOG_INFO, "set %s path checker for %s",
+		     checker_name, devname);
+		pp->checkfn = get_checker_addr(hwe->checker_index);
+		return 0;
 	}
-	syslog(LOG_INFO, "set readsector0 path checker for %s", devname);
+	syslog(LOG_INFO, "set readsector0 path checker for %s (default)",
+		devname);
 	return 0;
 }
 
@@ -170,7 +149,7 @@ exit_daemon (int status)
 	umount(CALLOUT_DIR);
 
 	syslog(LOG_INFO, "unlink pidfile");
-	unlink(PIDFILE);
+	unlink(DEFAULT_PIDFILE);
 
 	syslog(LOG_NOTICE, "--------shut down-------");
 	exit(status);
@@ -243,7 +222,7 @@ get_devmaps (void)
 			}
 			
 			if (!strncmp(target_type, "multipath", 9)) {
-				devmap = MALLOC(MAPNAMESIZE);
+				devmap = MALLOC(WWID_SIZE);
 				strcpy(devmap, names->name);
 				vector_alloc_slot(devmaps);
 				vector_set_slot(devmaps, devmap);
@@ -270,8 +249,8 @@ updatepaths (struct paths *allpaths, char *sysfs_path)
 	struct path *pp;
 	struct sysfs_directory * sdir;
 	struct sysfs_directory * devp;
-	char path[FILENAMESIZE];
-	char attr_path[FILENAMESIZE];
+	char path[FILE_NAME_SIZE];
+	char attr_path[FILE_NAME_SIZE];
 	char attr_buff[17];
 	
 	if (safe_sprintf(path, "%s/block", sysfs_path)) {
@@ -292,7 +271,7 @@ updatepaths (struct paths *allpaths, char *sysfs_path)
 	pthread_mutex_lock(allpaths->lock);
 
 	dlist_for_each_data(sdir->subdirs, devp, struct sysfs_directory) {
-		if (blacklist(devp->name)) {
+		if (blacklist(blist, devp->name)) {
 			syslog(LOG_DEBUG, "%s blacklisted", devp->name);
 			continue;
 		}
@@ -330,7 +309,7 @@ updatepaths (struct paths *allpaths, char *sysfs_path)
 		 */
 		pp = MALLOC(sizeof(struct path));
 
-		if (safe_snprintf(pp->dev_t, DEV_T_SIZE, "%s",
+		if (safe_snprintf(pp->dev_t, BLK_DEV_SIZE, "%s",
 				  attr_buff)) {
 			fprintf(stderr, "dev_t too small\n");
 			FREE(pp);
@@ -382,6 +361,51 @@ out:
 	return info.event_nr;
 }
 
+static void
+mark_failed_path (struct paths *allpaths, char *mapname)
+{
+	struct multipath *mpp;
+	struct pathgroup  *pgp;
+	struct path *pp;
+	struct path *app;
+	char *params, *status;
+	int i, j;
+
+	mpp = alloc_multipath();
+	if (!mpp)
+		return;
+
+	dm_get_map(mapname, &mpp->size, &params);
+	dm_get_status(mapname, &status);
+	
+	pthread_mutex_lock(allpaths->lock);
+	disassemble_map(allpaths->pathvec, params, mpp);
+	pthread_mutex_unlock(allpaths->lock);
+	
+	disassemble_status(status, mpp);
+
+	pthread_mutex_lock(allpaths->lock);
+	vector_foreach_slot (mpp->pg, pgp, i) {
+		vector_foreach_slot (pgp->paths, pp, j) {
+			if (pp->dmstate != PSTATE_FAILED)
+				continue;
+
+			app = find_path(allpaths->pathvec, pp->dev_t);
+			if (app && app->state != PATH_DOWN) {
+				syslog(LOG_NOTICE, "mark %s as failed",
+					pp->dev_t);
+				app->state = PATH_DOWN;
+			}
+		}
+	}
+	pthread_mutex_unlock(allpaths->lock);
+	free(params);
+	free(status);
+	free_multipath(mpp);
+
+	return;
+}
+
 static void *
 waitevent (void * et)
 {
@@ -418,19 +442,20 @@ out:
 
 	syslog(LOG_DEBUG, "%s", cmd);
 	syslog(LOG_NOTICE, "devmap event on %s", waiter->mapname);
+	mark_failed_path(waiter->allpaths, waiter->mapname);
 	execute_program(cmd, buff, 1);
 
 	/*
 	 * tell waiterloop we have an event
 	 */
-	pthread_mutex_lock (event_lock);
+	pthread_mutex_lock(event_lock);
 	pthread_cond_signal(event);
-	pthread_mutex_unlock (event_lock);
+	pthread_mutex_unlock(event_lock);
 	
 	/*
 	 * release waiter_lock so that waiterloop knows we are gone
 	 */
-	pthread_mutex_unlock (waiter->waiter_lock);
+	pthread_mutex_unlock(waiter->waiter_lock);
 	pthread_exit(waiter->thread);
 
 	return (NULL);
@@ -439,21 +464,21 @@ out:
 static void *
 waiterloop (void *ap)
 {
-	struct paths * allpaths;
+	struct paths *allpaths;
 	vector devmaps = NULL;
-	char * devmap;
+	char *devmap;
 	vector waiters;
 	struct event_thread *wp;
 	pthread_attr_t attr;
 	int r;
 	char buff[1];
 	int i, j;
-	char sysfs_path[FILENAMESIZE];
+	char sysfs_path[FILE_NAME_SIZE];
 
 	syslog(LOG_NOTICE, "start DM events thread");
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 
-	if (sysfs_get_mnt_path(sysfs_path, FILENAMESIZE)) {
+	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
 		syslog(LOG_ERR, "can not find sysfs mount point");
 		return NULL;
 	}
@@ -488,7 +513,7 @@ waiterloop (void *ap)
 		/*
 		 * update paths list
 		 */
-		syslog(LOG_INFO, "refresh failpaths list");
+		syslog(LOG_INFO, "refresh paths list");
 
 		while(updatepaths(allpaths, sysfs_path)) {
 			syslog(LOG_ERR, "can't update path list ... retry");
@@ -514,11 +539,12 @@ waiterloop (void *ap)
 			 */
 			if (j == VECTOR_SIZE(waiters)) {
 				wp = MALLOC (sizeof (struct event_thread));
-				strncpy (wp->mapname, devmap, MAPNAMESIZE);
+				strncpy (wp->mapname, devmap, WWID_SIZE);
 				wp->thread = MALLOC (sizeof (pthread_t));
 				wp->waiter_lock = (pthread_mutex_t *) 
 					MALLOC (sizeof (pthread_mutex_t));
 				pthread_mutex_init (wp->waiter_lock, NULL);
+				wp->allpaths = allpaths;
 				vector_alloc_slot (waiters);
 				vector_set_slot (waiters, wp);
 			}
@@ -646,68 +672,6 @@ initpaths (void)
 	return (allpaths);
 }
 
-#define VECTOR_ADDSTR(a, b) \
-	str = MALLOC (6 * sizeof(char)); \
-	snprintf(str, 6, b); \
-	vector_alloc_slot(a); \
-	vector_set_slot(a, str);
-
-static void
-setup_default_blist (vector blist)
-{
-	char * str;
-
-	VECTOR_ADDSTR(blist, "cciss");
-	VECTOR_ADDSTR(blist, "fd");
-	VECTOR_ADDSTR(blist, "hd");
-	VECTOR_ADDSTR(blist, "md");
-	VECTOR_ADDSTR(blist, "dm");
-	VECTOR_ADDSTR(blist, "sr");
-	VECTOR_ADDSTR(blist, "scd");
-	VECTOR_ADDSTR(blist, "st");
-	VECTOR_ADDSTR(blist, "ram");
-	VECTOR_ADDSTR(blist, "raw");
-	VECTOR_ADDSTR(blist, "loop");
-}
-
-#define VECTOR_ADDHWE(a, b, c, d) \
-	hwe = MALLOC (sizeof(struct hwentry)); \
-	hwe->vendor = MALLOC (9 * sizeof(char)); \
-	snprintf (hwe->vendor, 9, b); \
-	hwe->product = MALLOC (17 * sizeof(char)); \
-	snprintf (hwe->product, 17, c); \
-	hwe->checker_index = d; \
-	vector_alloc_slot(a); \
-	vector_set_slot(a, hwe);
-
-static void
-setup_default_hwtable (vector hwtable)
-{
-	struct hwentry * hwe;
-
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "HSV110 (C)COMPAQ", 0);
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "MSA1000", 0);
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "MSA1000 VOLUME", 0);
-	VECTOR_ADDHWE(hwtable, "DEC", "HSG80", 0);
-	VECTOR_ADDHWE(hwtable, "HP", "HSV110", 0);
-	VECTOR_ADDHWE(hwtable, "HP", "A6189A", 0);
-	VECTOR_ADDHWE(hwtable, "HP", "OPEN-", 0);
-	VECTOR_ADDHWE(hwtable, "DDN", "SAN DataDirector", 0);
-	VECTOR_ADDHWE(hwtable, "FSC", "CentricStor", 0);
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF400", 0);
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF500", 0);
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF600", 0);
-	VECTOR_ADDHWE(hwtable, "IBM", "ProFibre 4000R", 0);
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9100", 0);
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9300", 0);
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9400", 0);
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9500", 0);
-	VECTOR_ADDHWE(hwtable, "3PARdata", "VV", 0);
-	VECTOR_ADDHWE(hwtable, "STK", "OPENstorage D280", 0);
-	VECTOR_ADDHWE(hwtable, "SUN", "StorEdge 3510", 0);
-	VECTOR_ADDHWE(hwtable, "SUN", "T4", 0);
-}
-
 /*
  * this logic is all about keeping callouts working in case of
  * system disk outage (think system over SAN)
@@ -816,7 +780,7 @@ pidfile (pid_t pid)
 
 	buf = MALLOC(sizeof(struct stat));
 
-	if (!stat(PIDFILE, buf)) {
+	if (!stat(DEFAULT_PIDFILE, buf)) {
 		syslog(LOG_ERR, "already running : out");
 		FREE(buf);
 		exit(1);
@@ -830,7 +794,7 @@ pidfile (pid_t pid)
 		exit(1);
 	}
 	
-	file = fopen(PIDFILE, "w");
+	file = fopen(DEFAULT_PIDFILE, "w");
 	fprintf(file, "%d\n", pid);
 	fclose(file);
 	FREE(buf);
@@ -920,14 +884,16 @@ child (void * param)
 	allpaths = initpaths();
 	checkint = CHECKINT;
 
-	syslog(LOG_INFO, "read " CONFIGFILE);
-	init_data(CONFIGFILE, init_keywords);
+	syslog(LOG_INFO, "read " DEFAULT_CONFIGFILE);
+	init_data(DEFAULT_CONFIGFILE, init_keywords);
 
 	/*
 	 * fill the voids left in the config file
 	 */
-	if (binvec == NULL)
+	if (binvec == NULL) {
 		binvec = vector_alloc();
+		push_callout("/sbin/scsi_id");
+	}
 
 	if (multipath == NULL) {
 		multipath = MULTIPATH;
